@@ -12,8 +12,7 @@ from typing import (
     Generator,
     Iterable,
     List,
-    Mapping,
-    Optional,
+    Optional as O,
     Sequence,
     Tuple,
     TypeVar,
@@ -49,17 +48,17 @@ class Report:
 
     def __init__(
         self,
-        results: Mapping[Defn, E.ManagerResult],
+        results: Iterable[Tuple[Defn, E.ManagerResult]],
         filter_fn: Callable[[E.ManagerResult], bool] = lambda _: True,
     ):
-        self.results = results
+        self.results = list(results)
         self.filter_fn = filter_fn
 
     @property
     def code(self) -> int:
         return any(
             r
-            for r in self.results.values()
+            for _, r in self.results
             if isinstance(r, (E.ManagerError, E.InternalError)) and self.filter_fn(r)
         )
 
@@ -74,7 +73,7 @@ class Report:
         return '\n'.join(
             f'{_adorn_result(r)} {click.style(str(a), bold=True)}\n'
             + fill(r.message, initial_indent='  ', subsequent_indent='  ')
-            for a, r in self.results.items()
+            for a, r in self.results
             if self.filter_fn(r)
         )
 
@@ -94,12 +93,13 @@ class Report:
 
 
 class ManagerWrapper:
-    def __init__(self, debug: bool = False):
-        self.debug = debug
+    def __init__(self, ctx: click.Context):
+        self.ctx = ctx
 
     @cached_property
     def m(self) -> CliManager:
         import asyncio
+
         from .config import Config
         from .manager import CliManager, prepare_db_session
 
@@ -110,14 +110,13 @@ class ManagerWrapper:
 
         while True:
             try:
-                config = Config.read().ensure_dirs()
+                config = Config.read(profile=self.ctx.params['profile']).ensure_dirs()
             except FileNotFoundError:
-                ctx = click.get_current_context()
-                ctx.invoke(configure)
+                self.ctx.invoke(configure)
             else:
                 break
 
-        setup_logging(config.logger_dir, 'DEBUG' if self.debug else 'INFO')
+        setup_logging(config.logger_dir, 'DEBUG' if self.ctx.params['debug'] else 'INFO')
         db_session = prepare_db_session(config)
         manager = CliManager(config, db_session)
         return manager
@@ -152,7 +151,7 @@ def parse_into_defn(
             raise click.BadParameter(value)
         parts = (any_source, value)
     else:
-        parts = manager.decompose_url(value)
+        parts = manager.pair_url(value)
         if not parts:
             parts = value.partition(delim)[::2]
     return Defn(*parts)
@@ -166,14 +165,21 @@ def parse_into_defn_with_strategy(
     return list(map(Defn.with_strategy, defns, strategies))
 
 
+def parse_into_defn_with_version(
+    manager: CliManager, value: Sequence[Tuple[str, str]]
+) -> List[Defn]:
+    defns = parse_into_defn(manager, [d for _, d in value])
+    return list(map(Defn.with_version, defns, (v for v, _ in value)))
+
+
 def export_to_csv(pkgs: Sequence[models.Pkg], path: Path):
     "Export packages to CSV."
     import csv
 
     with Path(path).open('w', encoding='utf-8', newline='') as file:
         writer = csv.writer(file)
-        writer.writerow(('defn', 'strategy'))
-        writer.writerows((Defn.from_pkg(p), p.options.strategy) for p in pkgs)
+        writer.writerow(('defn', 'strategy', 'version'))
+        writer.writerows((Defn.from_pkg(p), p.options.strategy, p.version) for p in pkgs)
 
 
 def import_from_csv(manager: CliManager, path: Path) -> List[Defn]:
@@ -181,8 +187,9 @@ def import_from_csv(manager: CliManager, path: Path) -> List[Defn]:
     import csv
 
     with Path(path).open(encoding='utf-8', newline='') as contents:
-        rows = [(b, a) for a, b in islice(csv.reader(contents), 1, None)]
-    return parse_into_defn_with_strategy(manager, rows)
+        rows = islice(csv.reader(contents), 1, None)
+        defns = [Defn(*parse_into_defn(manager, d)[:2], Strategies[s], (v,)) for d, s, v in rows]
+    return defns
 
 
 def _callbackify(fn: Callable[..., _R]) -> Callable[[click.Context, Any, Any], _R]:
@@ -215,12 +222,13 @@ def _show_version(ctx: click.Context, param: Any, value: bool):
     is_eager=True,
     help='Show the version and exit.',
 )
+@click.option('--profile', '-p', default=None, help='Activate the specified profile.')
 @click.option('--debug', is_flag=True, default=False, help='Log more things.')
 @click.pass_context
-def main(ctx: click.Context, debug: bool):
+def main(ctx: click.Context, profile: O[str], debug: bool):
     "Add-on manager for World of Warcraft."
     if not ctx.obj:
-        ctx.obj = ManagerWrapper(debug)
+        ctx.obj = ManagerWrapper(ctx)
 
 
 @main.command()
@@ -237,13 +245,22 @@ def main(ctx: click.Context, debug: bool):
     '--with-strategy',
     '-s',
     multiple=True,
-    type=(click.Choice([s.name for s in Strategies]), str),
+    type=(click.Choice([s.name for s in Strategies.exposed()]), str),
     callback=_combine_into('addons', parse_into_defn_with_strategy),
     expose_value=False,
     metavar='<STRATEGY ADDON>...',
     help='A strategy followed by an add-on definition.  '
     'Available strategies are: '
-    f'{", ".join(repr(s.name) for s in islice(Strategies, 1, None))}.',
+    f'{", ".join(repr(s.name) for s in Strategies.exposed())}.',
+)
+@click.option(
+    '--version',
+    multiple=True,
+    type=(str, str),
+    callback=_combine_into('addons', parse_into_defn_with_version),
+    expose_value=False,
+    metavar='<VERSION ADDON>...',
+    help='A version followed by an add-on definition.',
 )
 @click.argument(
     'addons', nargs=-1, callback=_combine_into('addons', parse_into_defn), expose_value=False
@@ -253,11 +270,12 @@ def install(obj: M, addons: Sequence[Defn], replace: bool):
     "Install add-ons."
     if not addons:
         raise click.UsageError(
-            'You must provide at least one of "ADDONS", "--with-strategy" or "--import"'
+            'You must provide at least one of "ADDONS", "--with-strategy", '
+            '"--version" or "--import"'
         )
 
     results = obj.m.run(obj.m.install(addons, replace))
-    Report(results).generate_and_exit()
+    Report(results.items()).generate_and_exit()
 
 
 @main.command()
@@ -275,7 +293,7 @@ def update(obj: M, addons: Sequence[Defn]):
         defns = [Defn.from_pkg(p) for p in obj.m.db_session.query(models.Pkg).all()]
 
     results = obj.m.run(obj.m.update(defns))
-    Report(results, report_filter).generate_and_exit()
+    Report(results.items(), report_filter).generate_and_exit()
 
 
 @main.command()
@@ -284,17 +302,81 @@ def update(obj: M, addons: Sequence[Defn]):
 def remove(obj: M, addons: Sequence[Defn]):
     "Remove add-ons."
     results = obj.m.run(obj.m.remove(addons))
-    Report(results).generate_and_exit()
+    Report(results.items()).generate_and_exit()
+
+
+@main.command()
+@click.option(
+    '--undo',
+    is_flag=True,
+    default=False,
+    help='Undo rollback by reinstalling an add-on using the default strategy.',
+)
+@click.argument('addon', callback=_callbackify(parse_into_defn))
+@click.pass_context
+def rollback(ctx: click.Context, addon: Defn, undo: bool):
+    "Roll an add-on back to an older version."
+    from .prompts import Choice, select
+
+    manager = ctx.obj.m
+
+    pkg = manager.get_pkg(addon)
+    if not pkg:
+        Report([(addon, E.PkgNotInstalled())]).generate_and_exit()
+        return  # noop
+
+    resolver = manager.resolvers[pkg.source]
+    if not resolver.supports_rollback:
+        Report(
+            [(addon, E.PkgFileUnavailable('source does not support rollback'))]
+        ).generate_and_exit()
+
+    if undo:
+        Report(
+            chain(
+                manager.run(manager.remove([addon])).items(),
+                manager.run(manager.install([addon], replace=False)).items(),
+            )
+        ).generate_and_exit()
+
+    versions = (
+        manager.db_session.query(models.PkgVersionLog)
+        .filter(
+            models.PkgVersionLog.pkg_source == pkg.source, models.PkgVersionLog.pkg_id == pkg.id
+        )
+        .order_by(models.PkgVersionLog.install_time.desc())
+        .limit(10)
+        .all()
+    )
+    if len(versions) <= 1:
+        Report([(addon, E.PkgFileUnavailable('cannot find older versions'))]).generate_and_exit()
+
+    reconstructed_defn = Defn.from_pkg(pkg)
+    choices = [Choice([('', v.version)], disabled=(v.version == pkg.version)) for v in versions]
+    selection = select(
+        f'Select version of {reconstructed_defn} for rollback', choices
+    ).unsafe_ask()
+    Report(
+        chain(
+            manager.run(manager.remove([reconstructed_defn])).items(),
+            manager.run(
+                manager.install([reconstructed_defn.with_version(selection)], replace=False)
+            ).items(),
+        )
+    ).generate_and_exit()
 
 
 @main.command()
 @click.option(
     '--auto', '-a', is_flag=True, default=False, help='Do not ask for user confirmation.'
 )
+@click.option(
+    '--list-unreconciled', is_flag=True, default=False, help='List unreconciled add-ons and exit.'
+)
 @click.pass_context
-def reconcile(ctx: click.Context, auto: bool):
+def reconcile(ctx: click.Context, auto: bool, list_unreconciled: bool):
     "Reconcile pre-installed add-ons."
-    from .matchers import AddonFolder, match_toc_ids, match_toc_names, match_dir_names, get_folders
+    from .matchers import AddonFolder, get_folders, match_dir_names, match_toc_ids, match_toc_names
     from .models import is_pkg
     from .prompts import PkgChoice, confirm, select, skip
 
@@ -317,7 +399,7 @@ def reconcile(ctx: click.Context, auto: bool):
         '''
     )
 
-    manager: CliManager = ctx.obj.m
+    manager = ctx.obj.m
 
     def prompt_one(addons: List[AddonFolder], pkgs: List[models.Pkg]) -> Union[Defn, Tuple[()]]:
         def create_choice(pkg: models.Pkg):
@@ -360,10 +442,14 @@ def reconcile(ctx: click.Context, auto: bool):
             yield list(prompt(groups))
 
     leftovers = get_folders(manager)
-    if not leftovers:
+    if list_unreconciled:
+        table_rows = [('unreconciled',), *((f.name,) for f in sorted(leftovers))]
+        click.echo(tabulate(table_rows))
+        return
+    elif not leftovers:
         click.echo('No add-ons left to reconcile.')
         return
-    if not auto:
+    elif not auto:
         click.echo(preamble)
 
     matcher = match_all()
@@ -371,11 +457,11 @@ def reconcile(ctx: click.Context, auto: bool):
         selections = matcher.send(leftovers)
         if selections and (auto or confirm('Install selected add-ons?').unsafe_ask()):
             results = manager.run(manager.install(selections, replace=True))
-            Report(results).generate()
+            Report(results.items()).generate()
 
         leftovers = get_folders(manager)
 
-    if not auto and leftovers:
+    if leftovers:
         click.echo()
         table_rows = [('unreconciled',), *((f.name,) for f in sorted(leftovers))]
         click.echo(tabulate(table_rows))
@@ -397,7 +483,7 @@ def search(ctx: click.Context, limit: int, search_terms: str):
     "Search for add-ons to install."
     from .prompts import PkgChoice, checkbox, confirm
 
-    manager: CliManager = ctx.obj.m
+    manager = ctx.obj.m
 
     pkgs = manager.run(manager.search(search_terms, limit))
     if pkgs:
@@ -428,13 +514,13 @@ def search(ctx: click.Context, limit: int, search_terms: str):
     'addons', nargs=-1, callback=_callbackify(partial(parse_into_defn, raise_invalid=False))
 )
 @click.pass_obj
-def list_(obj: M, addons: Sequence[Defn], export: Optional[str], output_format: str):
+def list_(obj: M, addons: Sequence[Defn], export: O[str], output_format: str):
     "List installed add-ons."
     from sqlalchemy import and_, or_
 
     def format_deps(pkg: models.Pkg):
         deps = (Defn(pkg.source, d.id) for d in pkg.deps)
-        deps = (d.with_name(getattr(obj.m.get(d), 'slug', d.name)) for d in deps)
+        deps = (d.with_name(getattr(obj.m.get_pkg(d), 'slug', d.name)) for d in deps)
         return map(str, deps)
 
     def get_desc(pkg: models.Pkg):
@@ -466,13 +552,11 @@ def list_(obj: M, addons: Sequence[Defn], export: Optional[str], output_format:
         export_to_csv(pkgs, cast(Path, export))
     elif pkgs:
         if output_format == 'json':
-            from .models import MultiPkgModel
-
-            click.echo(MultiPkgModel.from_orm(pkgs).json(indent=2))
+            click.echo(models.MultiPkgModel.from_orm(pkgs).json(indent=2))
         elif output_format == 'detailed':
             formatter = click.HelpFormatter(max_width=99)
             for pkg in pkgs:
-                with formatter.section(Defn.from_pkg(pkg)):
+                with formatter.section(str(Defn.from_pkg(pkg))):
                     formatter.write_dl(
                         (
                             ('Name', pkg.name),
@@ -485,37 +569,11 @@ def list_(obj: M, addons: Sequence[Defn], export: Optional[str], output_format:
                             ('Options', f'{{"strategy": "{pkg.options.strategy}"}}'),
                         )
                     )
-
             click.echo(formatter.getvalue(), nl=False)
         else:
             click.echo('\n'.join(str(Defn.from_pkg(p)) for p in pkgs))
 
 
-@main.command()
-@click.option(
-    '--exclude-own', '-e', is_flag=True, default=False, help='Exclude folders managed by instawow.'
-)
-@click.option(
-    '--toc-entry',
-    '-t',
-    'toc_entries',
-    multiple=True,
-    help='An entry to extract from the TOC.  Repeatable.',
-)
-@click.pass_obj
-def list_folders(obj: M, exclude_own: bool, toc_entries: Sequence[str]) -> None:
-    "List add-on folders."
-    from .matchers import get_folders
-
-    folders = sorted(get_folders(obj.m, exclude_own=exclude_own))
-    if folders:
-        rows = [
-            ('unreconciled' if exclude_own else 'folder', *(f'[{e}]' for e in toc_entries)),
-            *((f.name, *(f.toc_reader[e].value for e in toc_entries)) for f in folders),
-        ]
-        click.echo(tabulate(rows))
-
-
 @main.command(hidden=True)
 @click.argument('addon', callback=_callbackify(partial(parse_into_defn, raise_invalid=False)))
 @click.pass_context
@@ -524,32 +582,18 @@ def info(ctx: click.Context, addon: Defn):
     ctx.invoke(list_, addons=(addon,), output_format='detailed')
 
 
-@main.command()
-@click.argument('addon', callback=_callbackify(partial(parse_into_defn, raise_invalid=False)))
-@click.pass_obj
-def visit(obj: M, addon: Defn):
-    "Open an add-on's homepage in your browser."
-    pkg = obj.m.get_from_substr(addon)
-    if pkg:
-        import webbrowser
-
-        webbrowser.open(pkg.url)
-    else:
-        Report({addon: E.PkgNotInstalled()}).generate_and_exit()
-
-
 @main.command()
 @click.argument('addon', callback=_callbackify(partial(parse_into_defn, raise_invalid=False)))
 @click.pass_obj
 def reveal(obj: M, addon: Defn):
-    "Open an add-on folder in your file manager."
-    pkg = obj.m.get_from_substr(addon)
+    "Bring an add-on up in your file manager."
+    pkg = obj.m.get_pkg_from_substr(addon)
     if pkg:
         import webbrowser
 
         webbrowser.open((obj.m.config.addon_dir / pkg.folders[0].name).as_uri())
     else:
-        Report({addon: E.PkgNotInstalled()}).generate_and_exit()
+        Report([(addon, E.PkgNotInstalled())]).generate_and_exit()
 
 
 @main.command()
@@ -560,11 +604,11 @@ def reveal(obj: M, addon: Defn):
     default=False,
     help='Show the active configuration and exit.',
 )
-@click.pass_obj
-def configure(obj: M, show_active: bool):
+@click.pass_context
+def configure(ctx: click.Context, show_active: bool):
     "Configure instawow."
     if show_active:
-        click.echo(obj.m.config.json(indent=2))
+        click.echo(ctx.obj.m.config.json(indent=2))
         return
 
     from prompt_toolkit.completion import WordCompleter
@@ -584,7 +628,9 @@ def configure(obj: M, show_active: bool):
         completer=WordCompleter(['retail', 'classic']),
         validator=PydanticValidator(Config, 'game_flavour'),
     )
-    config = Config(addon_dir=addon_dir, game_flavour=game_flavour).write()
+    config = Config(
+        addon_dir=addon_dir, game_flavour=game_flavour, profile=ctx.parent.params['profile'],
+    ).write()
     click.echo(f'Configuration written to: {config.config_file}')
 
 
@@ -618,23 +664,31 @@ def build_weakauras_companion(obj: M, account: str):
 @click.pass_obj
 def list_installed_wago_auras(obj: M, account: str):
     "List WeakAuras installed from Wago."
+    from textwrap import fill
+
     from .wa_updater import WaCompanionBuilder
 
     aura_groups = WaCompanionBuilder(obj.m, account).extract_installed_auras()
     installed_auras = sorted(
-        (Path(g.Meta.filename).stem, a.id, a.url, 'yes' if a.ignore_wago_update else 'no')
+        (
+            g.Meta.model.__name__,
+            fill(a.id, width=30, max_lines=1),
+            a.url,
+            'yes' if a.ignore_wago_update else 'no',
+        )
         for g in aura_groups
         for v in g.entries.values()
         for a in v
         if not a.parent
     )
-    click.echo(tabulate([('add-on', 'name', 'URL', 'ignore updates'), *installed_auras]))
+    click.echo(tabulate([('type', 'name', 'URL', 'ignore updates'), *installed_auras]))
 
 
 @main.command(hidden=True)
 @click.argument('filename', type=click.Path(dir_okay=False))
 def generate_catalogue(filename: str):
     import asyncio
+
     from .resolvers import MasterCatalogue
 
     catalogue = asyncio.run(MasterCatalogue.collate())
