@@ -34,10 +34,11 @@ from typing import (
 from loguru import logger
 
 from . import DB_REVISION, exceptions as E
-from .models import Pkg, PkgFolder, is_pkg
+from .models import Pkg, PkgFolder, PkgVersionLog, is_pkg
 from .resolvers import (
     CurseResolver,
     Defn,
+    GithubResolver,
     InstawowResolver,
     MasterCatalogue,
     Resolver,
@@ -58,6 +59,8 @@ if TYPE_CHECKING:
     import aiohttp
     from prompt_toolkit.shortcuts import ProgressBar
     from sqlalchemy.orm import scoped_session
+    from yarl import URL
+
     from .config import Config
 
     _T = TypeVar('_T')
@@ -141,7 +144,9 @@ async def download_archive(
     manager: Manager, pkg: Pkg, *, chunk_size: int = 4096
 ) -> ACM[_ArchiveR]:
     url = pkg.download_url
-    dst = manager.config.cache_dir / shasum(pkg.source, pkg.id, pkg.file_id)
+    dst = manager.config.cache_dir / shasum(
+        pkg.source, pkg.id, pkg.version, manager.config.game_flavour
+    )
 
     if await t(dst.exists)():
         pass
@@ -162,8 +167,10 @@ async def download_archive(
     return acquire_archive(dst)
 
 
-async def cache_json_response(manager: Manager, url: str, *args: Any, label: O[str] = None) -> Any:
-    dst = manager.config.cache_dir / shasum(url)
+async def cache_json_response(
+    manager: Manager, url: Union[str, URL], *args: Any, label: O[str] = None
+) -> Any:
+    dst = manager.config.cache_dir / shasum(str(url))
 
     if await t(is_not_stale)(dst, *args):
         text = await t(dst.read_text)(encoding='utf-8')
@@ -180,7 +187,8 @@ async def cache_json_response(manager: Manager, url: str, *args: Any, label: O[s
 
 def prepare_db_session(config: Config) -> scoped_session:
     from sqlalchemy import create_engine
-    from sqlalchemy.orm import sessionmaker, scoped_session
+    from sqlalchemy.orm import scoped_session, sessionmaker
+
     from .models import ModelBase, should_migrate
 
     db_path = config.config_dir / 'db.sqlite'
@@ -194,10 +202,11 @@ def prepare_db_session(config: Config) -> scoped_session:
     if should_migrate(engine, DB_REVISION):
         from alembic.command import stamp, upgrade
         from alembic.config import Config as AConfig
+
         from .utils import copy_resources
 
         with copy_resources(
-            f'{__package__}.migrations', f'{__package__}.migrations.versions'
+            f'{__package__}.migrations', f'{__package__}.migrations.versions',
         ) as tmp_dir:
             aconfig = AConfig()
             aconfig.set_main_option('script_location', str(tmp_dir / __package__ / 'migrations'))
@@ -251,12 +260,18 @@ class Manager:
     resolvers: Mapping[str, Resolver]
     catalogue: MasterCatalogue
 
+    resolver_classes = (
+        CurseResolver,
+        WowiResolver,
+        TukuiResolver,
+        GithubResolver,
+        InstawowResolver,
+    )
+
     def __init__(self, config: Config, db_session: scoped_session) -> None:
         self.config = config
         self.db_session = db_session
-
-        resolvers = (CurseResolver, WowiResolver, TukuiResolver, InstawowResolver)
-        self.resolvers = _ResolverDict({r.source: r(self) for r in resolvers})
+        self.resolvers = _ResolverDict((r.source, r(self)) for r in self.resolver_classes)
         self.catalogue = None  # type: ignore
 
     @property
@@ -267,46 +282,48 @@ class Manager:
     def web_client(self, value: aiohttp.ClientSession) -> None:
         _web_client.set(value)
 
-    def get(self, defn: Defn) -> O[Pkg]:
-        "Get a package from (source, id) or (source, slug)."
+    async def _consume_seq(
+        self, coros_by_defn: Dict[Defn, Callable[..., Awaitable[Any]]]
+    ) -> Dict[Defn, E.ManagerResult]:
+        return {d: await E.ManagerResult.acapture(c()) for d, c in coros_by_defn.items()}
+
+    def get_pkg(self, defn: Defn) -> O[Pkg]:
+        "Retrieve an installed package from a definition."
         return (
             self.db_session.query(Pkg)
             .filter(Pkg.source == defn.source, (Pkg.id == defn.name) | (Pkg.slug == defn.name))
             .first()
         )
 
-    def get_from_substr(self, defn: Defn) -> O[Pkg]:
-        "Get a package from a partial slug."
-        return self.get(defn) or (
+    def get_pkg_from_substr(self, defn: Defn) -> O[Pkg]:
+        "Retrieve an installed package from a partial slug."
+        return self.get_pkg(defn) or (
             self.db_session.query(Pkg)
             .filter(Pkg.slug.contains(defn.name))
             .order_by(Pkg.name)
             .first()
         )
 
-    def decompose_url(self, url: str) -> O[Tuple[str, str]]:
-        "Parse a URL into a source–name tuple."
-        for resolver in self.resolvers.values():
-            name = resolver.decompose_url(url)
-            if name:
-                return resolver.source, name
+    def pair_url(self, url: str) -> O[Tuple[str, str]]:
+        "Attempt to pair a URL with a resolver, returning the source and name."
+        return next(
+            filter(
+                all, ((r.source, r.get_name_from_url(url)) for r in self.resolvers.values())
+            ),  # type: ignore
+            None,
+        )
 
     async def synchronise(self) -> None:
         "Fetch the master catalogue from the interwebs."
         if self.catalogue is None:
             label = 'Synchronising master catalogue'
             url = (
-                'https://raw.githubusercontent.com/layday/instascrape/data/'
+                'https://raw.githubusercontent.com/layday/instawow-data/data/'
                 'master-catalogue-v1.compact.json'
             )  # v1
             catalogue = await cache_json_response(self, url, 4, 'hours', label=label)
             self.catalogue = MasterCatalogue.parse_obj(catalogue)
 
-    async def _consume_seq(
-        self, coros_by_defn: Dict[Defn, Callable[..., Awaitable[Any]]]
-    ) -> Dict[Defn, E.ManagerResult]:
-        return {d: await E.ManagerResult.acapture(c()) for d, c in coros_by_defn.items()}
-
     async def _resolve_deps(self, results: Iterable[Any]) -> Dict[Defn, Any]:
         """Resolve package dependencies.
 
@@ -345,9 +362,10 @@ class Manager:
         return results_by_defn
 
     async def search(self, search_terms: str, limit: int) -> Dict[Defn, Pkg]:
-        "Search the combined names catalogue for packages."
+        "Search the master catalogue for packages by name."
         import heapq
         import string
+
         from jellyfish import jaro_winkler
 
         trans_table = str.maketrans(dict.fromkeys(string.punctuation, ' '))
@@ -393,16 +411,22 @@ class Manager:
                 await trash([self.config.addon_dir / f for f in folders], self.config.temp_dir)
             await extract(self.config.addon_dir)
 
-        pkg.folders = [PkgFolder(name=f) for f in folders]
-        self.db_session.add(pkg)
-        self.db_session.commit()
+            pkg.folders = [PkgFolder(name=f) for f in folders]
+            self.db_session.add(pkg)
+            self.db_session.merge(
+                PkgVersionLog(version=pkg.version, pkg_source=pkg.source, pkg_id=pkg.id)
+            )
+            self.db_session.commit()
+
         return E.PkgInstalled(pkg)
 
     async def install(self, defns: Sequence[Defn], replace: bool) -> Dict[Defn, E.ManagerResult]:
-        prelim_results = await self.resolve([d for d in defns if not self.get(d)], with_deps=True)
+        prelim_results = await self.resolve(
+            [d for d in defns if not self.get_pkg(d)], with_deps=True
+        )
         # Weed out installed deps - this isn't super efficient but avoids
         # having to deal with local state in the resolver
-        results = {d: r for d, r in prelim_results.items() if not self.get(d)}
+        results = {d: r for d, r in prelim_results.items() if not self.get_pkg(d)}
         installables = {(d, r): download_archive(self, r) for d, r in results.items() if is_pkg(r)}
         archives = await gather(installables.values())
 
@@ -432,18 +456,21 @@ class Manager:
             await self.remove_one(old_pkg)
             await extract(self.config.addon_dir)
 
-        pkg.folders = [PkgFolder(name=f) for f in folders]
-        self.db_session.add(pkg)
-        self.db_session.commit()
+            pkg.folders = [PkgFolder(name=f) for f in folders]
+            self.db_session.add(pkg)
+            self.db_session.merge(
+                PkgVersionLog(version=pkg.version, pkg_source=pkg.source, pkg_id=pkg.id)
+            )
+            self.db_session.commit()
+
         return E.PkgUpdated(old_pkg, pkg)
 
     async def update(self, defns: Sequence[Defn]) -> Dict[Defn, E.ManagerResult]:
         # Rebuild ``Defn`` with ID and strategy from package for defns
         # of installed packages.  Using the ID has the benefit of resolving
-        # installed-but-renamed packages
-        maybe_pkgs = ((d, self.get(d)) for d in defns)
+        # installed-but-renamed packages - the slug is transient; the ID isn't
+        maybe_pkgs = ((d, self.get_pkg(d)) for d in defns)
         checked_defns = {Defn.from_pkg(p) if p else c: p for c, p in maybe_pkgs}
-
         # Results can contain errors
         # installables are those results which are packages
         # and updatables are packages with updates
@@ -452,7 +479,7 @@ class Manager:
         updatables = {
             (d, checked_defns[d], p): download_archive(self, p)
             for d, p in installables.items()
-            if p.file_id != cast(Pkg, checked_defns[d]).file_id
+            if p.version != cast(Pkg, checked_defns[d]).version
         }
         archives = await gather(updatables.values())
 
@@ -473,10 +500,11 @@ class Manager:
         )
         self.db_session.delete(pkg)
         self.db_session.commit()
+
         return E.PkgRemoved(pkg)
 
     async def remove(self, defns: Sequence[Defn]) -> Dict[Defn, E.ManagerResult]:
-        pkgs_by_defn = ((d, self.get(d)) for d in defns)
+        pkgs_by_defn = ((d, self.get_pkg(d)) for d in defns)
         coros = dict_chain(
             defns,
             partial(_error_out, E.PkgNotInstalled()),
@@ -500,6 +528,7 @@ async def cancel_tickers() -> AsyncIterator[None]:
 
 def init_cli_web_client(*, Bar: ProgressBar) -> aiohttp.ClientSession:
     from cgi import parse_header
+
     from aiohttp import TraceConfig, hdrs
 
     def extract_filename(response: aiohttp.ClientResponse) -> str:
@@ -529,11 +558,8 @@ def init_cli_web_client(*, Bar: ProgressBar) -> aiohttp.ClientSession:
             try:
                 bar = Bar(label=label, total=total)
                 content = params.response.content
-
                 while not content.is_eof():
-                    # This is ``bar.current`` in prompt_toolkit v2
-                    # and ``.items_completed`` in v3
-                    bar.current = bar.items_completed = content.total_bytes
+                    bar.items_completed = content.total_bytes
                     Bar.invalidate()
                     await asyncio.sleep(_tick_interval)
             finally:
@@ -551,11 +577,12 @@ def init_cli_web_client(*, Bar: ProgressBar) -> aiohttp.ClientSession:
 
 class CliManager(Manager):
     def run(self, awaitable: Awaitable[_T]) -> _T:
-        async def run():
-            async with init_cli_web_client(Bar=Bar) as self.web_client, cancel_tickers():
-                return await awaitable
-
         with make_progress_bar() as Bar:
+
+            async def run():
+                async with init_cli_web_client(Bar=Bar) as self.web_client, cancel_tickers():
+                    return await awaitable
+
             loop = asyncio.new_event_loop()
             try:
                 return loop.run_until_complete(run())
