@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from datetime import datetime
 import enum
-from itertools import count, takewhile
+from itertools import chain, count, takewhile
 import re
 from typing import (
     TYPE_CHECKING,
@@ -10,10 +10,12 @@ from typing import (
     AsyncIterable,
     ClassVar,
     Dict,
+    Iterable,
     List,
     NamedTuple,
     Optional as O,
     Set,
+    Tuple,
     cast,
 )
 
@@ -43,28 +45,36 @@ class Strategies(enum.Enum):
     curse_latest_beta = enum.auto()
     curse_latest_alpha = enum.auto()
     any_flavour = enum.auto()
+    version = enum.auto()
+
+    @classmethod
+    def exposed(cls) -> Iterable[Strategies]:
+        return (s for s in cls if s not in {cls.default, cls.version})
 
 
 class Defn(NamedTuple):
     source: str
     name: str
     strategy: Strategies = Strategies.default
+    strategy_vals: Tuple[str, ...] = ()
     source_id: O[str] = None
 
     @classmethod
     def from_pkg(cls, pkg: m.Pkg) -> Defn:
-        return cls(
-            pkg.source,
-            pkg.slug,
-            Strategies[pkg.options.strategy],  # type: ignore
-            pkg.id,
-        )
+        strategy = Strategies[pkg.options.strategy]
+        strategy_vals = (pkg.version,) if strategy is Strategies.version else ()
+        return cls(pkg.source, pkg.slug, strategy, strategy_vals, pkg.id)
 
     def with_name(self, name: str) -> Defn:
-        return self.__class__(self.source, name, self.strategy, self.source_id)
+        return self.__class__(self.source, name, self.strategy, self.strategy_vals, self.source_id)
 
     def with_strategy(self, strategy: Strategies) -> Defn:
-        return self.__class__(self.source, self.name, strategy, self.source_id)
+        return self.__class__(self.source, self.name, strategy, (), self.source_id)
+
+    def with_version(self, version: str) -> Defn:
+        return self.__class__(
+            self.source, self.name, Strategies.version, (version,), self.source_id
+        )
 
     def __str__(self) -> str:
         return f'{self.source}:{self.name}'
@@ -88,6 +98,7 @@ class MasterCatalogue(BaseModel):
     @classmethod
     async def collate(cls) -> MasterCatalogue:
         from types import SimpleNamespace
+
         from .manager import Manager, init_web_client
 
         resolvers = (CurseResolver, WowiResolver, TukuiResolver)
@@ -112,20 +123,32 @@ class Resolver(ManagerAttrAccessMixin):
         self.manager = manager
 
     def __init_subclass__(cls) -> None:
-        async def wrapper(self: Resolver, defn: Defn, *args: Any, **kwargs: Any) -> m.Pkg:
+        async def resolve_wrapper(
+            self: Resolver, defn: Defn, metadata: O[JsonDict] = None
+        ) -> m.Pkg:
             if defn.strategy in self.strategies:
-                return await resolve_one(self, defn, *args, **kwargs)
+                return await resolve_one(self, defn, metadata)  # type: ignore
             raise E.PkgStrategyUnsupported(defn.strategy)
 
         resolve_one = cls.resolve_one
-        cls.resolve_one = wrapper
+        cls.resolve_one = resolve_wrapper  # type: ignore
+
+    @property
+    def supports_rollback(self) -> bool:
+        "Whether the resolver supports rollback operations."
+        return Strategies.version in self.strategies
 
     @staticmethod
-    def decompose_url(value: str) -> O[str]:
-        "Attempt to extract definition names from resolvable URLs."
+    def get_name_from_url(value: str) -> O[str]:
+        "Attempt to extract a definition name from a given URL."
 
     async def resolve(self, defns: List[Defn]) -> Dict[Defn, Any]:
         "Resolve add-on definitions into packages."
+        results = await gather(self.resolve_one(d) for d in defns)
+        return dict(zip(defns, results))
+
+    async def resolve_one(self, defn: Defn, metadata: O[JsonDict] = None) -> m.Pkg:
+        "Resolve an individual definition into a package."
         raise NotImplementedError
 
     async def collect_items(self) -> AsyncIterable[_CItem]:
@@ -143,13 +166,14 @@ class CurseResolver(Resolver):
         Strategies.curse_latest_beta,
         Strategies.curse_latest_alpha,
         Strategies.any_flavour,
+        Strategies.version,
     }
 
     # Reference: https://twitchappapi.docs.apiary.io/
     addon_api_url = URL('https://addons-ecs.forgesvc.net/api/v2/addon')
 
     @staticmethod
-    def decompose_url(value: str) -> O[str]:
+    def get_name_from_url(value: str) -> O[str]:
         url = URL(value)
         if url.host == 'www.wowace.com' and len(url.parts) > 2 and url.parts[1] == 'projects':
             return url.parts[2].lower()
@@ -181,68 +205,82 @@ class CurseResolver(Resolver):
         if not metadata:
             raise E.PkgNonexistent
 
-        files = metadata['latestFiles']
-        if not files:
-            raise E.PkgFileUnavailable('no files available for download')
+        if defn.strategy is Strategies.version:
 
-        def is_not_libless(f: JsonDict):
-            # There's also an 'isAlternate' field that's missing some
-            # 50 lib-less files from c. 2008.  'exposeAsAlternative' is
-            # absent from the /file endpoint
-            return not f['exposeAsAlternative']
+            from .manager import cache_json_response
 
-        if defn.strategy is Strategies.any_flavour:
+            files = await cache_json_response(
+                self.manager,
+                self.addon_api_url / str(metadata['id']) / 'files',
+                3600,
+                label=f'Fetching metadata from {self.name}',
+            )
 
-            def is_compatible_with_game_version(f: JsonDict):
-                return True
+            def is_compatible(f: JsonDict):
+                return defn.strategy_vals[0] == f['displayName']
 
         else:
-            classic_version_prefix = '1.13'
-            flavour = 'wow_classic' if self.config.is_classic else 'wow_retail'
 
-            def is_compatible_with_game_version(f: JsonDict):
-                # Files can belong both to retail and classic
-                # but ``gameVersionFlavor`` can only be one of
-                # 'wow_retail' or 'wow_classic'.  To spice things up,
-                # ``gameVersion`` might not be populated so we still have to
-                # check the value of ``gameVersionFlavor``
-                return f['gameVersionFlavor'] == flavour or any(
-                    v.startswith(classic_version_prefix) is self.config.is_classic
-                    for v in f['gameVersion']
+            files = metadata['latestFiles']
+
+            def is_not_libless(f: JsonDict):
+                # There's also an 'isAlternate' field that's missing from some
+                # 50 lib-less files from c. 2008.  'exposeAsAlternative' is
+                # absent from the /file endpoint
+                return not f['exposeAsAlternative']
+
+            if defn.strategy is Strategies.any_flavour:
+
+                def supports_game_version(f: JsonDict):
+                    return True
+
+            else:
+                classic_version_prefix = '1.13'
+                flavour = 'wow_classic' if self.config.is_classic else 'wow_retail'
+
+                def supports_game_version(f: JsonDict):
+                    # Files can belong both to retail and classic
+                    # but ``gameVersionFlavor`` can only be one of
+                    # 'wow_retail' or 'wow_classic'.  To spice things up,
+                    # ``gameVersion`` might not be populated so we still have to
+                    # check the value of ``gameVersionFlavor``
+                    return f['gameVersionFlavor'] == flavour or any(
+                        v.startswith(classic_version_prefix) is self.config.is_classic
+                        for v in f['gameVersion']
+                    )
+
+            # 1 = stable; 2 = beta; 3 = alpha
+            if defn.strategy is Strategies.latest:
+
+                def is_of_specified_quality(f: JsonDict):
+                    return True
+
+            elif defn.strategy is Strategies.curse_latest_beta:
+
+                def is_of_specified_quality(f: JsonDict):
+                    return f['releaseType'] == 2
+
+            elif defn.strategy is Strategies.curse_latest_alpha:
+
+                def is_of_specified_quality(f: JsonDict):
+                    return f['releaseType'] == 3
+
+            else:
+
+                def is_of_specified_quality(f: JsonDict):
+                    return f['releaseType'] == 1
+
+            def is_compatible(f: JsonDict):
+                return (
+                    is_not_libless(f) and supports_game_version(f) and is_of_specified_quality(f)
                 )
 
-        # 1 = stable; 2 = beta; 3 = alpha
-        if defn.strategy is Strategies.latest:
-
-            def has_release_type(f: JsonDict):
-                return True
-
-        elif defn.strategy is Strategies.curse_latest_beta:
-
-            def has_release_type(f: JsonDict):
-                return f['releaseType'] == 2
-
-        elif defn.strategy is Strategies.curse_latest_alpha:
-
-            def has_release_type(f: JsonDict):
-                return f['releaseType'] == 3
-
-        else:
-
-            def has_release_type(f: JsonDict):
-                return f['releaseType'] == 1
-
+        if not files:
+            raise E.PkgFileUnavailable('no files available for download')
         try:
             file = max(
-                (
-                    f
-                    for f in files
-                    if is_not_libless(f)
-                    and is_compatible_with_game_version(f)
-                    and has_release_type(f)
-                ),
-                # The ``id`` is just a counter so we don't have to
-                # go digging around dates
+                filter(is_compatible, files),
+                # The ``id`` is just a counter so we don't have to go digging around dates
                 key=lambda f: f['id'],
             )
         except ValueError:
@@ -266,7 +304,6 @@ class CurseResolver(Resolver):
             name=metadata['name'],
             description=metadata['summary'],
             url=metadata['websiteUrl'],
-            file_id=file['id'],
             download_url=file['downloadUrl'],
             date_published=file['fileDate'],
             version=file['displayName'],
@@ -341,7 +378,7 @@ class WowiResolver(Resolver):
     _files: O[JsonDict] = None
 
     @staticmethod
-    def decompose_url(value: str) -> O[str]:
+    def get_name_from_url(value: str) -> O[str]:
         url = URL(value)
         if (
             url.host in {'wowinterface.com', 'www.wowinterface.com'}
@@ -369,7 +406,7 @@ class WowiResolver(Resolver):
                 self.manager,
                 self.list_api_url,
                 3600,
-                label=f'Synchronising {self.source} catalogue',
+                label=f'Synchronising {self.name} catalogue',
             )
             self._files = {i['UID']: i for i in files}
 
@@ -393,12 +430,8 @@ class WowiResolver(Resolver):
         if not metadata:
             raise E.PkgNonexistent
 
-        # Files "pending" are downloadable but do not have a checksum which
-        # we use as a unique file identifier
-        # TODO: investigate using a different field for updates
-        if metadata['UIPending'] == '1':
-            raise E.PkgFileUnavailable('new file awaiting approval')
-
+        # 'UIPending' is set to '1' for files awaiting approval during which
+        # time 'UIMD5' is null - all other fields appear to be filled correctly
         return m.Pkg(
             source=self.source,
             id=metadata['UID'],
@@ -406,7 +439,6 @@ class WowiResolver(Resolver):
             name=metadata['UIName'],
             description=metadata['UIDescription'],
             url=metadata['UIFileInfoURL'],
-            file_id=metadata['UIMD5'],
             download_url=metadata['UIDownload'],
             date_published=metadata['UIDate'],
             version=metadata['UIVersion'],
@@ -436,7 +468,7 @@ class TukuiResolver(Resolver):
     retail_uis = {'-1': 'tukui', '-2': 'elvui', 'tukui': 'tukui', 'elvui': 'elvui'}
 
     @staticmethod
-    def decompose_url(value: str) -> O[str]:
+    def get_name_from_url(value: str) -> O[str]:
         url = URL(value)
         if url.host == 'www.tukui.org' and url.path in {
             '/addons.php',
@@ -447,11 +479,7 @@ class TukuiResolver(Resolver):
             if name:
                 return name
 
-    async def resolve(self, defns: List[Defn]) -> Dict[Defn, Any]:
-        results = await gather(self.resolve_one(d) for d in defns)
-        return dict(zip(defns, results))
-
-    async def resolve_one(self, defn: Defn) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: ... = None) -> m.Pkg:
         name = ui_name = self.retail_uis.get(defn.name)
         if not name:
             name = ''.join(takewhile('-'.__ne__, defn.name))
@@ -476,7 +504,6 @@ class TukuiResolver(Resolver):
             name=addon['name'],
             description=addon['small_desc'],
             url=addon['web_url'],
-            file_id=addon['version'],
             download_url=addon['url'],
             date_published=datetime.fromisoformat(addon['lastupdate']),
             version=addon['version'],
@@ -512,6 +539,99 @@ class TukuiResolver(Resolver):
                     )
 
 
+class GithubResolver(Resolver):
+    source = 'github'
+    name = 'GitHub'
+    strategies = {
+        Strategies.default,
+        Strategies.version,
+    }
+
+    repos_api_url = URL('https://api.github.com/repos')
+
+    @staticmethod
+    def get_name_from_url(value: str) -> O[str]:
+        url = URL(value)
+        if url.host == 'github.com' and len(url.parts) > 2:
+            return '/'.join(url.parts[1:3])
+
+    async def resolve_one(self, defn: Defn, metadata: ... = None) -> m.Pkg:
+        """Resolve a hypothetical add-on hosted on GitHub.
+
+        The GitHub resolver, ahem, 'builds' on the work done by Torkus
+        (see https://github.com/ogri-la/strongbox/blob/develop/github-addons.md) -
+        it purports to support the different kinds of add-ons supported by strongbox
+        with the exception that instawow does not look for TOC files or validate
+        the contents of the ZIP file.
+        instawow will attempt to prioritise add-ons compatible with your selected
+        game flavour.  It will otherwise install the first file it encounters.
+        instawow will only install assets attached to releases.  It will not
+        install add-ons from VCS tarballs or 'zipballs' (i.e. from source).
+        """
+        from aiohttp import ClientResponseError
+
+        from .manager import cache_json_response
+
+        repo_url = self.repos_api_url / defn.name
+        try:
+            project_metadata = await cache_json_response(self.manager, repo_url, 3600)
+        except ClientResponseError as error:
+            if error.status == 404:
+                raise E.PkgNonexistent
+            raise
+
+        if defn.strategy is Strategies.version:
+            release_url = repo_url / 'releases/tags' / defn.strategy_vals[0]
+        else:
+            release_url = repo_url / 'releases/latest'
+        async with self.web_client.get(release_url) as response:
+            if response.status == 404:
+                raise E.PkgFileUnavailable('release not found')
+            release_metadata = await response.json()
+
+        try:
+
+            def is_valid_asset(asset: JsonDict):
+                return (
+                    # There is something of a convention that Classic archives
+                    # end in '-classic' and lib-less archives end in '-nolib'.
+                    # Archives produced by packager follow this convention
+                    not asset['name'].endswith(('-nolib.zip', '-nolib-classic.zip'))
+                    and asset['content_type']
+                    in {'application/zip', 'application/x-zip-compressed'}
+                    # A failed upload has a state value of 'starter'
+                    and asset['state'] == 'uploaded'
+                )
+
+            assets = release_metadata.get('assets', [])
+            matching_asset = next(
+                chain(
+                    (
+                        a
+                        for a in assets
+                        if is_valid_asset(a)
+                        and a['name'].endswith('-classic.zip') is self.config.is_classic
+                    ),
+                    filter(is_valid_asset, assets),
+                )
+            )
+        except StopIteration:
+            raise E.PkgFileUnavailable
+
+        return m.Pkg(
+            source=self.source,
+            id=project_metadata['full_name'],
+            slug=project_metadata['full_name'].lower(),
+            name=project_metadata['name'],
+            description=project_metadata['description'],
+            url=project_metadata['html_url'],
+            download_url=matching_asset['browser_download_url'],
+            date_published=release_metadata['published_at'],
+            version=release_metadata['tag_name'],
+            options=m.PkgOptions(strategy=defn.strategy.name),
+        )
+
+
 class InstawowResolver(Resolver):
     source = 'instawow'
     name = 'instawow'
@@ -519,11 +639,7 @@ class InstawowResolver(Resolver):
 
     _addons = {('0', 'weakauras-companion'), ('1', 'weakauras-companion-autoupdate')}
 
-    async def resolve(self, defns: List[Defn]) -> Dict[Defn, Any]:
-        results = await gather(self.resolve_one(d) for d in defns)
-        return dict(zip(defns, results))
-
-    async def resolve_one(self, defn: Defn) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: ... = None) -> m.Pkg:
         try:
             id_, slug = next(p for p in self._addons if defn.name in p)
         except StopIteration:
@@ -546,7 +662,6 @@ class InstawowResolver(Resolver):
             name='WeakAuras Companion',
             description='A WeakAuras Companion clone.',
             url='https://github.com/layday/instawow',
-            file_id=checksum,
             download_url=builder.addon_file.as_uri(),
             date_published=datetime.now(),
             version=checksum[:7],
