@@ -12,6 +12,7 @@ from typing import (
     TYPE_CHECKING,
     AbstractSet,
     Any,
+    AsyncIterator,
     Awaitable,
     Callable,
     Dict,
@@ -38,6 +39,7 @@ Literal = _Literal  # ...
 
 if TYPE_CHECKING:
     from prompt_toolkit.shortcuts import ProgressBar
+
     from .manager import CliManager
 
     _H = TypeVar('_H', bound=Hashable)
@@ -112,12 +114,12 @@ def bucketise(iterable: Iterable[_V], key: Callable[[_V], _H] = lambda v: v) ->
     bucket: Any = defaultdict(list)
     for value in iterable:
         bucket[key(value)].append(value)
-    return dict(bucket)
+    return bucket
 
 
 def dict_chain(
-    keys: Iterable[_H], default: Any, *overrides: Iterable[Tuple[_H, Any]]
-) -> Dict[_H, Any]:
+    keys: Iterable[_H], default: Any, *overrides: Iterable[Tuple[_H, _V]]
+) -> Dict[_H, _V]:
     "Construct a dictionary from a series of iterables with overlapping keys."
     return dict(chain(zip(keys, repeat(default)), *overrides))
 
@@ -153,7 +155,32 @@ async def gather(it: Iterable[Awaitable[_V]], return_exceptions: bool = True) ->
 
 
 def run_in_thread(fn: Callable[..., _V]) -> Callable[..., Awaitable[_V]]:
-    return lambda *a, **k: asyncio.get_running_loop().run_in_executor(None, lambda: fn(*a, **k))
+    def wrapper(*args: Any, **kwargs: Any):
+        loop = asyncio.get_running_loop()
+        return loop.run_in_executor(None, lambda: fn(*args, **kwargs))
+
+    return wrapper
+
+
+class _StopIteration(Exception):
+    pass
+
+
+def _iit_next(it: Iterator[_V]) -> _V:
+    try:
+        return next(it)
+    except StopIteration:
+        raise _StopIteration
+
+
+# From Starlette: https://github.com/encode/starlette/blob/78f7095/starlette/concurrency.py
+async def iter_in_thread(it: Iterator[_V]) -> AsyncIterator[_V]:
+    loop = asyncio.get_running_loop()
+    while True:
+        try:
+            yield await loop.run_in_executor(None, _iit_next, it)
+        except _StopIteration:
+            return
 
 
 @contextmanager
@@ -205,7 +232,7 @@ def tabulate(rows: Sequence[Sequence[Any]], *, max_col_width: int = 60) -> str:
     rows = [tuple(map(apply_max_col_width, r)) for r in rows]
     head, *tail = rows
 
-    base_template = ' '.join(f'{{{{{{0}}{w}}}}}' for w in calc_resultant_col_widths(rows))
+    base_template = '  '.join(f'{{{{{{0}}{w}}}}}' for w in calc_resultant_col_widths(rows))
     row_template = base_template.format(':<')
     table = '\n'.join(
         (
@@ -220,6 +247,7 @@ def tabulate(rows: Sequence[Sequence[Any]], *, max_col_width: int = 60) -> str:
 def make_progress_bar(**kwargs: Any) -> ProgressBar:
     "A ``ProgressBar`` with download progress expressed in megabytes."
     import signal
+
     from prompt_toolkit.formatted_text import HTML
     from prompt_toolkit.shortcuts.progress_bar import ProgressBar, formatters
     from prompt_toolkit.utils import Event
@@ -257,7 +285,7 @@ def make_progress_bar(**kwargs: Any) -> ProgressBar:
                 return f'{value / 2 ** 20:.1f}'
 
             return HTML(self.template).format(
-                current=format_pct(progress.current), total=format_pct(progress.total)
+                current=format_pct(progress.items_completed), total=format_pct(progress.total)
             )
 
     f = [
