@@ -1,6 +1,6 @@
 import pytest
 
-import instawow.exceptions as E
+from instawow import exceptions as E
 from instawow.models import Pkg
 from instawow.resolvers import Defn, Strategies
 
@@ -9,11 +9,11 @@ from instawow.resolvers import Defn, Strategies
 @pytest.mark.parametrize(
     'strategy', [Strategies.default, Strategies.latest, Strategies.any_flavour]
 )
-async def test_resolve_curse_pkgs(manager, request, strategy):
+async def test_resolve_curse_simple_pkgs(manager, request, strategy):
     results = await manager.resolve(
         [
             Defn('curse', 'tomcats', strategy),
-            Defn('curse', 'method-dungeon-tools', strategy),
+            Defn('curse', 'mythic-dungeon-tools', strategy),
             Defn('curse', 'classiccastbars', strategy),
             Defn('curse', 'elkbuffbars', strategy),
         ]
@@ -52,8 +52,20 @@ async def test_resolve_curse_pkgs(manager, request, strategy):
 
 @pytest.mark.asyncio
 async def test_resolve_curse_latest_pkg(manager):
-    (latest,) = (await manager.resolve([Defn('curse', 'tomcats', Strategies.latest)])).values()
-    assert isinstance(latest, Pkg)
+    (latest_pkg,) = (await manager.resolve([Defn('curse', 'tomcats', Strategies.latest)])).values()
+    assert latest_pkg.options.strategy == 'latest'
+
+
+@pytest.mark.asyncio
+async def test_resolve_curse_versioned_pkg(manager):
+    (versioned_pkg,) = (
+        await manager.resolve(
+            [Defn('curse', 'molinari', Strategies.version, ('70300.51-Release',))]
+        )
+    ).values()
+    assert (
+        versioned_pkg.options.strategy == 'version' and versioned_pkg.version == '70300.51-Release'
+    )
 
 
 @pytest.mark.asyncio
@@ -66,6 +78,20 @@ async def test_resolve_curse_deps(manager):
     assert ['mechagon-rare-share', 'rare-share'] == [d.slug for d in with_deps.values()]
 
 
+@pytest.mark.asyncio
+async def test_resolve_wowi_pkgs(manager):
+    results = await manager.resolve(
+        [Defn('wowi', '13188-molinari'), Defn('wowi', '13188', Strategies.latest)]
+    )
+    either, invalid = results.values()
+
+    assert isinstance(either, Pkg)
+    assert (
+        isinstance(invalid, E.PkgStrategyUnsupported)
+        and invalid.message == "'latest' strategy is not valid for source"
+    )
+
+
 @pytest.mark.asyncio
 async def test_resolve_tukui_pkgs(manager):
     results = await manager.resolve(
@@ -94,14 +120,31 @@ async def test_resolve_tukui_pkgs(manager):
 
 
 @pytest.mark.asyncio
-async def test_resolve_wowi_pkgs(manager):
+async def test_resolve_github_pkgs(manager):
     results = await manager.resolve(
-        [Defn('wowi', '13188-molinari'), Defn('wowi', '13188', Strategies.latest)]
+        [
+            Defn('github', 'AdiAddons/AdiButtonAuras'),
+            Defn('github', 'AdiAddons/AdiButtonAuras', Strategies.version, ('2.1.0',)),
+            Defn('github', 'AdiAddons/AdiButtonAuras', Strategies.version, ('2.0.19',)),
+            Defn('github', 'WeakAuras/WeakAuras2'),
+            Defn('github', 'p3lim-wow/Molinari'),
+            Defn('github', 'layday/foo-bar'),
+        ]
     )
-    either, invalid = results.values()
-
-    assert isinstance(either, Pkg)
+    (
+        lib_and_nolib,
+        older_version,
+        assetless,
+        retail_and_classic,
+        releaseless,
+        missing,
+    ) = results.values()
+    assert 'nolib' not in lib_and_nolib.download_url
+    assert older_version.options.strategy == 'version' and older_version.version == '2.1.0'
+    assert isinstance(assetless, E.PkgFileUnavailable)
     assert (
-        isinstance(invalid, E.PkgStrategyUnsupported)
-        and invalid.message == "'latest' strategy is not valid for source"
+        isinstance(releaseless, E.PkgFileUnavailable)
+        and releaseless.message == 'release not found'
     )
+    assert ('classic' in retail_and_classic.download_url) is manager.config.is_classic
+    assert isinstance(missing, E.PkgNonexistent)
