@@ -1,7 +1,26 @@
 import pytest
 from yarl import URL
 
-from instawow.wa_updater import ApiMetadata, WaCompanionBuilder, WeakAura, WeakAuras
+from instawow.wa_updater import WaCompanionBuilder, WeakAura, WeakAuras
+
+
+@pytest.fixture
+def wa_saved_vars(manager):
+    saved_vars = (
+        manager.config.addon_dir.parents[1] / 'WTF' / 'Account' / 'test' / 'SavedVariables'
+    )
+    saved_vars.mkdir(parents=True)
+    (saved_vars / WeakAuras.Meta.filename).write_text(
+        '''\
+WeakAurasSaved = {
+    ["displays"] = {
+        ["Foo"] = {
+            ["bar"] = "baz",
+        },
+    },
+}
+'''
+    )
 
 
 @pytest.fixture
@@ -9,22 +28,6 @@ def builder(manager):
     yield WaCompanionBuilder(manager, 'test')
 
 
-@pytest.mark.skip
-@pytest.mark.asyncio
-@pytest.mark.parametrize('ids', [['RaidCDs', 'bfaraid2'], ['bfaraid2', 'RaidCDs']])
-async def test_id_order_is_retained_in_aura_metadata(builder, ids):
-    results = await builder.get_wago_aura_metadata(ids)
-    assert ids == [r.slug for r in results]
-
-
-@pytest.mark.skip
-@pytest.mark.asyncio
-async def test_id_length_is_retained_in_aura_metadata(builder):
-    ids = ['bfaraid2', 'foobar', 'RaidCDs']  # Invalid ID flanked by two valid IDs
-    results = await builder.get_wago_aura_metadata(ids)
-    assert [ApiMetadata, type(None), ApiMetadata] == [type(r) for r in results]
-
-
 def test_can_parse_empty_displays_table(builder):
     assert (
         builder.extract_auras(
@@ -112,10 +115,15 @@ WeakAurasSaved = {
     )
 
 
-def test_can_build_addon_without_updates(builder):
+def test_can_build_addon_with_empty_seq(builder):
     builder.make_addon([])
 
 
+@pytest.mark.asyncio
+async def test_can_build_addon_with_mock_saved_vars(builder, wa_saved_vars):
+    await builder.build()
+
+
 def test_build_is_reproducible(builder):
     builder.make_addon([])
     checksum = builder.checksum()
